% Computation of a dual frame of a frame G based on Prop. 2.1 in the paper 
% "On compactly supported dual frames of Gabor frames"
%(finite dimensional case, general frames)

function [TGdnew] = dualFiniteDimCaseGeneralFr(n,TG,TGd,TW)
%function [TGdnew] = dualFiniteDimCaseGeneralFr(n,p,TG,TGd,TW)

% Author:
% Diana Stoeva, dtstoeva@kfs.oeaw.ac.at
% created May 2020, final version Feb 2021
%
% This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
% To view a copy of this license, visit 
% http://creativecommons.org/licenses/by-nc-sa/3.0/
% and
% https://creativecommons.org/licenses/by-nc-sa/3.0/legalcode

%% Input arguments of the function

% n - number of frame vectors;
% TG - the synthesis matrix corresponding to a given frame G (G=(g_1,...,g_n)), 
%      its size is pxn (p rows and n columns), where p denotes the
%      dimension of the space (each vector - with p coordinates)
% TGd - the synthesis matrix of a given dual frame Gd of G
% TW - the synthesis matrix of a Bessel sequence W

%% Initialization 
TGdnew=TGd + TW;

%% Determining a new dual frame GDnew of G 

for k=1:n
    for j=1:n
     TGdnew(:,k)=TGdnew(:,k)-dot(TGd(:,k),TG(:,j)).*TW(:,j);
        % dot(f,g) - returns the scalar dot product of two vectors f and g 
        % X(:,k) - gives the entire k-th column of a matrix X
    end    
end


%% Output
% TGdnew - the sysnthesis matrix of the new dual frame Gdnew of G

end