% Implementation of Prop. 3.1 from the paper "On compactly supported dual frames of Gabor frames".
% It concerns approximation of the canonical dual of a frame G via dual frames on every iteration,
% with precision of the first 4 digits after the decimal dot. 
%(finite dimensional case, general frames)

% For Ex.3.2 and Fig.2 in the paper, we run this function with the respective input frames G and Gd,
% see below which program-lines to be marked and which ones to be unmarked. 

function [CanDual,broi] = dualFiniteDimCaseGeneralFr_Canonical_Run

% Author:
% Diana Stoeva, dtstoeva@kfs.oeaw.ac.at
% created May 2020, final version Feb 2021
%
% This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
% To view a copy of this license, visit 
% http://creativecommons.org/licenses/by-nc-sa/3.0/
% and
% https://creativecommons.org/licenses/by-nc-sa/3.0/legalcode

%% input

% n - number of frame vectors;
% p - dimension of the space (each vector - with p coordinates); 
% TG - the synthesis matrix corresponding to a given frame G (G=(g_1,...,g_n)),
%      its size is pxn (p rows and n columns)
% TGd - the synthesis matrix of a given dual frame Gd of G
% TW - the synthesis matrix of a bessel sequence W

clear;

%%%%%%% RUN 1, Ex. 3.2(i) with lambda=1/3, Fig.2(a)
%%% To run the script for the setting in Ex. 3.2(i) with lambda=1/3,
%%% unmark the next three command-lines (and mark all the other command-lines in the input-paragraph)
 run=1
 [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1 
 lambda=1/3; 
%%%%%%% This run will give output broi=1, i.e. 1st iteration = 2nd iteration,
%%%%%%% the output-figure named FigApproxCanonicalDualRun_1.png is the graphix visualized in Fig.2(a)

%%%%%%% RUN 2, Ex. 3.2(i) with lambda=1/4, Fig.2(b)
%%% To run the script for the setting in Ex. 3.2(i) with lambda=1/4,
%%% unmark the next three command-lines (and mark all the other command-lines in the input-paragraph)
% run=2
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1 
% lambda=1/4; 
%%%%%%% This run will give output broi=7, i.e. 7th iter = 8th iter
%%%%%%% the output-figure named FigApproxCanonicalDualRun_2.png is the graphix visualized in Fig.2(b)

%%%%%%% RUN 6, Ex. 3.2(i) with lambda=1/2=2/(A^{opt}+B^{opt}),  A^{opt}=1, B^{opt}=3
%%% To run the script for the setting in Ex. 3.2(i) with lambda=1/2, 
%%% unmark the next three command-lines (and mark all the other command-lines in the input-paragraph)
% run=6
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1 
% lambda=1/2;  
%%%%%%% This run will give output broi=14, i.e. 14th iter=15th iter



%%%%% RUN 3, Ex. 3.2(ii) with lambda=1/3, Fig.2(c)
%%% To run the script for the setting in Ex. 3.2(ii) with lambda=1/3, 
%%% unmark the next three command-lines (and mark all the other command-lines in the input-paragraph)
% run=3
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1_2 
% lambda=1/3; 
%%%%%%% This run will give output broi=1, i.e. 1st iteration=2nd iter.
%%%%%%% the output-figure named FigApproxCanonicalDualRun_3.png is the graphix visualized in Fig.2(c)

%%%%% RUN 9, Ex. 3.2(ii) with lambda=1/2
%%% To run the script for the setting in Ex. 3.2(ii) with lambda=1/2, 
%%% unmark the next three command-lines (and mark all the other command-lines in the input-paragraph)
% run=9
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1_2 
% lambda=1/2; 
%%%%%%% This run will give output broi=15, i.e. 15th iter. = 16th iter. 

%%%%% RUN 10, Ex. 3.2(ii) with lambda=1/4
%%% To run the script for the setting in Ex. 3.2(ii) with lambda=1/4, 
%%% unmark the next three command-lines (and mark all the other command-lines in the input-paragraph)
% run=10
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1_2 
% lambda=1/4; 
%%%%%%% This run will give output broi=8, i.e. 8th iter. = 9th iter. 
 


%%%%% RUN 4, Ex. 3.2(iii) with lambda=1/3, Fig.2(d)
% run=4
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1_3 
% lambda=1/3; 
%%%%%%% This run will give output broi=21, i.e. 21st iter. = 22nd iter. 
%%%%%%% the output-figure named FigApproxCanonicalDualRun_4.png is the graphix visualized in Fig.2(d)

%%%%% RUN 11, Ex. 3.2(iii) with lambda=1/4
% run=11
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1_3 
% lambda=1/4; 
%%%%%%% This run will give output broi=29, i.e. 29th iter. = 30th iter. 

%%%%% RUN 12, Ex. 3.2(iii) with lambda=1/2
% run=12
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input1_3 
% lambda=1/2; 
%%%%%%% This run will give output broi=15, i.e. 15th iter. = 16th iter. 



%%%%% RUN 5, Ex. 3.2(iv) with lambda=2/9
% run=5
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input2
% lambda=2/9; 
%%%%%%% This run will give output broi=32, i.e. 32nd iter. = 33rd iter. 

%%%%% RUN 7, Ex. 3.2(iv) with lambda=1/4, Fig.2(f)
%%% the graphic in Run 5 looks little bit better, but Run 7 is with less iterations, so maybe to use Run 7
% run=7
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input2
% lambda=1/4; 
%%%%%%% This run will give output broi=28, i.e. 28th iter. = 29th iter. 
%%%%%%% the output-figure named FigApproxCanonicalDualRun_7.png is the graphix visualized in Fig.2(f)
 
%%%%% RUN 8, Ex. 3.2(iv) with lambda=2/7=2/(A^{opt}+B^{opt}, A^{opt}=1, B^{opt}=6, Fig.2(e)
% run=8
% [n,p,TG,TGd]=dualFiniteDimCaseGeneralFr_Input2
% lambda=2/7 
%%%%%%% This run will give output broi=31, i.e. 31st iter. = 32nd iter. 
%%%%%%% the output-figure named FigApproxCanonicalDualRun_8.png is the graphix visualized in Fig.2(e)
 
%% Main part of the script, approximation of the canonical dual 
% We will create an array of matrices in order to save 
% all the dual frames in the algorithm until we get 
% the approximation of the canonical dual with the desired precision.
% This is the way - first we define a matrix, e.g. A=T1
% then we add to it a second matrix via A(:,:,2)= T2, and so on. 

br=1;
TW=lambda.*TG;

% first new dual by dualFiniteDimCaseGeneralFr(n,TG,TGd,TW)
TGnewduals=dualFiniteDimCaseGeneralFr(n,TG,TGd,TW);
Cut4digits = fix(TGnewduals*10^4)/(10^4);

v=1;
while (v==1)
    br=br+1;
    TGnewduals(:,:,br)=dualFiniteDimCaseGeneralFr(n,TG,TGnewduals(:,:,br-1),TW);
    Cut4digits(:,:,br)=fix(TGnewduals(:,:,br)*10^4)/(10^4);
    if isequal(Cut4digits(:,:,br),Cut4digits(:,:,br-1)) %this means the 4 digits after the dot are the same at the steps br and br-1
    v=0;
    end
end

%% output

CanDual=TGnewduals(:,:,br-1); %this is the desired approximation of the canonical dual from the iteration step br-1
broi=br-1

%check
check=TG*transpose(TGnewduals(:,:,br-1))

%% plot of the given frame and the following dual frames:
% the given initial dual frame, the dual frame from from the first iteration step (if broi>1), 
% and the dual frame which approximates the canonical dual with the desired precision (from step broi)

figure
      
% plot of the given frame in green 
for k = 1 : n  
quiver( 0,0,TG(1,k),TG(2,k),0, 'g' )
vectorNameG = sprintf('g_%d', k); % at each step %d will be replaced with the respective value of k
text(TG(1,k)+1/55,TG(2,k)-1/7,vectorNameG,'Color', 'g')
hold on
end

for k = 1 : n

% plot of the initial dual frame in blue
quiver(0,0,TGd(1,k),TGd(2,k),0,'b--') 
vectorNameGd = sprintf('g^d_%d', k); 
if run==4
text(TGd(1,k),TGd(2,k)+1/11,vectorNameGd, 'Color', 'b')
else
text(TGd(1,k)+1/55,TGd(2,k)+1/9,vectorNameGd, 'Color', 'b')
end
hold on


format1 = "f^{%d}_{%d}";

% if broi>1, plot of the dual frame from the first iteration step in red 
if broi>1
TGdbr1=TGnewduals(:,:,1); % the dual frame at the first iteration step br=1
quiver(0,0,TGdbr1(1,k),TGdbr1(2,k),0, 'r--' ) 
vectorNameGd = sprintf(format1,1,k);
 if (run==2) & (k==2) 
  vectorNameGdnew = sprintf('= %s', vectorNameGd); 
  text(TGdbr1(1,k)+6/55,TGdbr1(2,k)+0.3/9,vectorNameGdnew, 'Color', 'r')
 elseif or(run==5,run==7) & (k==3)
  text(TGdbr1(1,k)-1/15,TGdbr1(2,k)+1/9,vectorNameGd, 'Color', 'r')
 elseif (run==6) 
  text(TGdbr1(1,k)-1/15,TGdbr1(2,k)+1/9,vectorNameGd, 'Color', 'r')
 else
  text(TGdbr1(1,k)+1/55,TGdbr1(2,k)+0.3/9,vectorNameGd, 'Color', 'r')
 end
hold on
end

% plot of the dual frame which approximates the canonical dual with the desired precision (from step broi) in magenta 
quiver( 0,0,CanDual(1,k),CanDual(2,k),0, 'm' ) 
vectorNameGdnew=sprintf(format1,br-1,k);
vectorNameGdnew2 = sprintf('_%d', k); 
 if or(run==1,run==2)
  text(CanDual(1,k)+1/15,CanDual(2,k),vectorNameGdnew, 'Color', 'm');
  text(CanDual(1,k)+2.1/15,CanDual(2,k), '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
  text(CanDual(1,k)+3/15+2/25,CanDual(2,k),vectorNameGdnew2, 'Color', 'm');
 elseif run==3 
     if k==3
      text(CanDual(1,k)+1/15-1/5,CanDual(2,k)-1/5,vectorNameGdnew, 'Color', 'm');
      text(CanDual(1,k)+2.1/15-1/5,CanDual(2,k)-1/5, '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
      text(CanDual(1,k)+3/15+3/25-1/5,CanDual(2,k)-1/5,vectorNameGdnew2, 'Color', 'm');
     else
      text(CanDual(1,k)+1/15,CanDual(2,k),vectorNameGdnew, 'Color', 'm');
      text(CanDual(1,k)+2.1/15,CanDual(2,k), '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
      text(CanDual(1,k)+3/15+3/25,CanDual(2,k),vectorNameGdnew2, 'Color', 'm');
     end
 elseif run==4
     if k==3
      text(CanDual(1,k)-1.7/5,CanDual(2,k)-1/15,vectorNameGdnew, 'Color', 'm');
      text(CanDual(1,k)+2.1/15-1.7/5,CanDual(2,k)-1/15, '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
      text(CanDual(1,k)+3/15+3/25-1.7/5,CanDual(2,k)-1/15,vectorNameGdnew2, 'Color', 'm');
     else
      text(CanDual(1,k)+1/15-1/21,CanDual(2,k),vectorNameGdnew, 'Color', 'm');
      text(CanDual(1,k)+2.5/15,CanDual(2,k), '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
      text(CanDual(1,k)+3/15+3.5/25,CanDual(2,k),vectorNameGdnew2, 'Color', 'm');
     end
 else
    if or(run==5,run==7) & (k==3)
    text(CanDual(1,k)+0.3/15-1.1/5,CanDual(2,k)-1/11,vectorNameGdnew, 'Color', 'm');
    text(CanDual(1,k)+2.45/15-1.1/5,CanDual(2,k)-1.5/11, '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
    text(CanDual(1,k)+3/15+3.5/25-1.1/5,CanDual(2,k)-1.5/11,vectorNameGdnew2, 'Color', 'm');
    elseif run==8
        if k==3
         text(CanDual(1,k)+0.3/15-1/15-1/10,CanDual(2,k)+0.5/25-1/6,vectorNameGdnew, 'Color', 'm');
         text(CanDual(1,k)+2.45/15-1/15-1/30-1/10,CanDual(2,k)-1/6, '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
         text(CanDual(1,k)+3/15+3.5/25-1/15-1/10,CanDual(2,k)-1/6,vectorNameGdnew2, 'Color', 'm');
        else
         text(CanDual(1,k)+0.3/15-1/15,CanDual(2,k)+1.5/25+1/20,vectorNameGdnew, 'Color', 'm');
         text(CanDual(1,k)+2.45/15-1/15-1/30,CanDual(2,k)+1/25+1/20, '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
         text(CanDual(1,k)+3/15+3.5/25-1/15,CanDual(2,k)+1/25+1/20,vectorNameGdnew2, 'Color', 'm');
        end
    else
        text(CanDual(1,k)+0.3/15-1/15,CanDual(2,k)+1.5/25+1/15,vectorNameGdnew, 'Color', 'm');
        text(CanDual(1,k)+2.45/15-1/15,CanDual(2,k)+1/25+1/15, '$$\approx\tilde{g}$$', 'Interpreter', 'LaTeX','Color', 'm' );
        text(CanDual(1,k)+3/15+3.5/25-1/15,CanDual(2,k)+1/25+1/15,vectorNameGdnew2, 'Color', 'm');
    end
 end
hold on
end

% set limits of the axes
    if run==1 || run==2 || run==6
     Xmin=-1.5; Xmax= 1.5; Ymin=-0.5; Ymax=1.5;
    elseif run==3 || run==9 || run==10
     Xmin=-1.5; Xmax= 2.5; Ymin=-1.5; Ymax=2.5; 
    elseif run==11 || run==12
     Xmin=-1.5; Xmax= 2.5; Ymin=-0.5; Ymax=1.5; 
    elseif run==4
     Xmin=-1.5; Xmax= 2.5; Ymin=-0.5; Ymax=1.5; 
    elseif run==8
     Xmin=-2.3; Xmax= 2.5; Ymin=-0.5; Ymax=2.3;
    else 
     Xmin=-2.3; Xmax= 1.7; Ymin=-0.5; Ymax=2.3;
    end

%hold on
axis([Xmin Xmax Ymin Ymax])
hold on
grid on

hold off



% To save the figure
baseFileName = sprintf('FigApproxCanonicalDualRun_%d', run) 
savefig(baseFileName)
saveas(gcf, baseFileName, 'png' );

end
